@@ -12,7 +12,7 @@ def main():
     parser.add_argument('-c', '--configuration-directory', help='Path to configuration directory',
                         default=os.path.join(os.environ['HOME'], '.config', 'dcnnt'))
     parser.add_argument('mode', choices=('doc', 'foreground', 'start', 'stop', 'restart'), nargs='?', default='start',
-                        help='Mode to run programm: doc - just print config documentation and exit, '
+                        help='Mode to run program: doc - just print config documentation and exit, '
                              'foreground - run programm in current tty, '
                              'start/stop/restart - run and stop programm as daemon')
     args = parser.parse_args(sys.argv[1:])
