@@ -2,6 +2,7 @@ from .base import PluginInitializer
 from .file_transfer import FileTransferPlugin
 from .remote_commands import RemoteCommandsPlugin
 from .notifications import NotificationsPlugin
+from .opener import OpenerPlugin
 
 
-PLUGINS = FileTransferPlugin, RemoteCommandsPlugin, NotificationsPlugin
+PLUGINS = FileTransferPlugin, OpenerPlugin, RemoteCommandsPlugin, NotificationsPlugin
