@@ -1,5 +1,6 @@
 import glob
-import logging
+from abc import ABC
+from logging import Logger, DEBUG, INFO, WARNING, ERROR
 
 from ..common import *
 
@@ -37,6 +38,30 @@ class PluginInitializer:
         return False
 
 
+class HandlerExit(Exception):
+    """Raising of this exception causes RPC handler exit and normal response sending"""
+
+    def __init__(self, response: RPCResponse):
+        self.response = response
+        self.message = response.result.get('message', str(response.result))
+
+    @classmethod
+    def new(cls, request: RPCRequest, code: int, message: str):
+        """Create typical HandlerExit with code-message result response"""
+        return cls(RPCResponse(request.id, dict(code=code, message=message)))
+
+
+class HandlerFail(Exception):
+    """Raising of this exception causes RPC handler exit, no any response sent"""
+
+    def __init__(self, message: str):
+        self.message = message
+
+
+class PluginFail(HandlerFail):
+    """Raising of this exception causes plugin loop exit, no any response sent"""
+
+
 class Plugin:
     """Base plugin class"""
     MARK = b'\0\0\0\0'
@@ -49,9 +74,9 @@ class Plugin:
     def __init__(self, app, handler, device):
         self.app, self.logger, self.handler, self.sock, self.device = app, app.log, handler, handler.sock, device
 
-    def log(self, message, level=logging.INFO):
+    def log(self, message, level: int = INFO):
         """Make log record with specified level"""
-        self.logger.log(level, '[{}] {}'.format(self.NAME, message))
+        self.logger.log(level, f'[{self.NAME}] {message}')
 
     @classmethod
     def post_init(cls):
@@ -72,7 +97,7 @@ class Plugin:
             except (KeyError, IndexError):
                 return
             except TypeError:
-                self.log('TypeError in plugin {} while config value {}'.format(self.NAME, path), logging.ERROR)
+                self.log('TypeError in plugin {} while config value {}'.format(self.NAME, path), ERROR)
                 return
         return node
 
@@ -98,21 +123,73 @@ class Plugin:
         request_raw = self.read()
         if request_raw is None:
             return
-        self.log(request_raw, logging.DEBUG)
+        self.log(request_raw, DEBUG)
         try:
             return RPCRequest.from_dict(json.loads(request_raw.decode()))
         except BaseException as e:
-            self.log(e, logging.WARNING)
+            self.log(e, WARNING)
 
-    def rpc_send(self, obj):
+    def rpc_send(self, obj: RPCObject):
         """Send JSON-RPC 2.0 response, notification or request"""
         try:
             serialized_obj = json.dumps(obj.to_dict())
             self.send(serialized_obj.encode())
-            self.log("Sent: {}".format(serialized_obj), logging.DEBUG)
+            self.log("Sent: {}".format(serialized_obj), DEBUG)
         except BaseException as e:
-            self.log(e, logging.WARNING)
+            self.log(e, WARNING)
+
+    def process_request(self, request: RPCRequest):
+        """Process one RPC request"""
+        raise NotImplementedError
 
     def main(self):
         """Entry point for plugins called in TCP handler"""
-        raise NotImplementedError
+        while True:
+            request = self.rpc_read()
+            self.log(request)
+            if request is None:
+                self.log('No more requests, stop handler')
+                return
+            try:
+                self.process_request(request)
+            except HandlerExit as e:
+                self.log(f'Handler exit: {e.message}')
+                self.rpc_send(e.response)
+            except PluginFail as e:
+                self.log(f'Plugin fail: {e.message}')
+                return
+            except HandlerFail as e:
+                self.log(f'Handler fail: {e.message}')
+            except BaseException as e:
+                self.logger.error(f'Exception {e}')
+                self.logger.exception(e)
+                return
+
+
+class BaseFilePlugin(Plugin, ABC):
+    """Common option for files with file transfer support"""
+
+    def receive_file(self, request: RPCRequest, download_directory: str) -> str:
+        """Receive and save file from client device"""
+        try:
+            name, size = request.params['name'], request.params['size']
+        except KeyError as e:
+            raise HandlerFail(f'KeyError {e}')
+        path = os.path.join(download_directory, name)
+        self.log(f'Receiving {size} bytes to file {path}')
+        self.rpc_send(RPCResponse(request.id, dict(code=0, message='OK')))
+        wrote = 0
+        with open(path, 'wb') as f:
+            while wrote < size:
+                buf = self.read()
+                if buf is None:
+                    raise HandlerFail(f'File receiving aborted ({wrote} bytes received)')
+                if len(buf) == 0:
+                    req = self.rpc_read()
+                    if req.method == "cancel":
+                        raise HandlerExit.new(request, 1, 'Canceled')
+                wrote += len(buf)
+                f.write(buf)
+        self.log(f'File received ({wrote} bytes)', INFO)
+        self.rpc_send(RPCResponse(request.id, dict(code=0, message='OK')))
+        return path
