@@ -2,11 +2,11 @@ import fnmatch
 import logging
 import subprocess
 
-from .base import Plugin
+from .base import BaseFilePlugin, HandlerExit, HandlerFail
 from ..common import *
 
 
-class FileTransferPlugin(Plugin):
+class FileTransferPlugin(BaseFilePlugin):
     """Receive file from phone"""
     MARK = b'file'
     NAME = 'FileTransferPlugin'
@@ -28,7 +28,8 @@ class FileTransferPlugin(Plugin):
     PART = 65532
     shared_files_index = list()
 
-    def check_file_filter(self, path: str, glob_str: str) -> bool:
+    @staticmethod
+    def check_file_filter(path: str, glob_str: str) -> bool:
         """Check if file allowed for sharing by filter"""
         return fnmatch.fnmatch(path, glob_str)
 
@@ -49,7 +50,7 @@ class FileTransferPlugin(Plugin):
             elif os.path.isfile(path):
                 if self.check_file_filter(path, filter_data):
                     self.shared_files_index.append(path)
-                    index=len(self.shared_files_index) - 1
+                    index = len(self.shared_files_index) - 1
                     res.append(dict(name=name, node_type='file', size=os.path.getsize(path), index=index))
         return res
 
@@ -74,42 +75,16 @@ class FileTransferPlugin(Plugin):
             res.append(dict(name=name, node_type='directory', size=len(dir_list), children=dir_list))
         return res
 
-    def handle_upload(self, request):
+    def handle_upload(self, request: RPCRequest):
         """Receive and save file from client"""
-        try:
-            name, size = request.params['name'], request.params['size']
-        except KeyError as e:
-            self.log('KeyError {}'.format(e), logging.WARN)
-        else:
-            path = os.path.join(self.conf('download_directory'), name)
-            self.log('Receiving {} bytes to file {}'.format(size, path))
-            self.rpc_send(RPCResponse(request.id, dict(code=0, message='OK')))
-            f = open(path, 'wb')
-            wrote = 0
-            while wrote < size:
-                buf = self.read()
-                if buf is None:
-                    self.log('File receiving aborted ({} bytes received)'.format(wrote), logging.WARN)
-                    return
-                if len(buf) == 0:
-                    req = self.rpc_read()
-                    if req.method == "cancel":
-                        self.log('File receiving canceled by client ({} bytes received)'.format(wrote), logging.INFO)
-                        f.close()
-                        self.rpc_send(RPCResponse(request.id, dict(code=1, message='Canceled')))
-                        return
-                wrote += len(buf)
-                f.write(buf)
-            f.close()
-            self.log('File received ({} bytes)'.format(wrote), logging.INFO)
-            self.rpc_send(RPCResponse(request.id, dict(code=0, message='OK')))
-            on_download = self.conf('on_download')
-            if isinstance(on_download, str):
-                command = on_download.format(path=path)
-                self.log('Execute: "{}"'.format(command))
-                subprocess.call(command, shell=True)
+        path = self.receive_file(request, self.conf('download_directory'))
+        on_download = self.conf('on_download')
+        if isinstance(on_download, str):
+            command = on_download.format(path=path)
+            self.log('Execute: "{}"'.format(command))
+            subprocess.call(command, shell=True)
 
-    def handle_list_shared(self, request):
+    def handle_list_shared(self, request: RPCRequest):
         """Create shared files info and return as JSON"""
         try:
             result = self.shared_files_info()
@@ -147,16 +122,10 @@ class FileTransferPlugin(Plugin):
             else:
                 self.rpc_send(RPCResponse(request.id, dict(code=1, message='No such index: {}'.format(index))))
 
-    def main(self):
-        while True:
-            request = self.rpc_read()
-            self.log(request)
-            if request is None:
-                self.log('[FileTransferPlugin] No more requests, stop handler')
-                return
-            if request.method == 'list':
-                self.handle_list_shared(request)
-            elif request.method == 'download':
-                self.handle_download(request)
-            elif request.method == 'upload':
-                self.handle_upload(request)
+    def process_request(self, request: RPCRequest):
+        if request.method == 'list':
+            self.handle_list_shared(request)
+        elif request.method == 'download':
+            self.handle_download(request)
+        elif request.method == 'upload':
+            self.handle_upload(request)
